import os
import sys
import json
from .terminal import FontColor
from .messages import PARAMS_REQUIRED

BASE_DIR = ''


def get_config(file_path: str):
    """
    get config data
    :param file_path: config file path
    :return: dict that contains config data
    """
    config_path = os.path.join(BASE_DIR, file_path)
    with open(config_path, 'r') as file_reader:
        config = json.loads(file_reader.read())
    return config


def get_settings_filename():
    """
    return settings filename from cli params
    :return: config filename
    """
    return sys.argv[1]


def get_settings():
    """
    return settings data
    :return: dict that contains config data
    """
    return get_config(get_settings_filename())


def get_params_filename():
    """
    return user algorithm params filename
    :return: params filename
    """
    if sys.argv[2] == 'None':
        # TODO: replace with logger
        print(FontColor.failed(PARAMS_REQUIRED))
        exit(0)

    return sys.argv[2]


def get_params():
    """
    return user algorithm params data
    :return: dict that contains params data
    """
    return get_config(get_params_filename())



import os

from .access_logger import AccessLogger
from .error_logger import ErrorLogger

from algomax_common.messages import INVALID_LOG_PATH, CLASS_IS_SINGLETON
from algomax_common.terminal import FontColor


DEFAULT_LOG_PATH = 'logs'


class AMLogger:
    """
    custom json logger
    this class is singleton
    """
    shared = None

    # default log path
    path = DEFAULT_LOG_PATH

    def __init__(self):
        if AMLogger.shared is not None:
            raise Exception(CLASS_IS_SINGLETON.format('AMLogger'))
        else:
            AMLogger.shared = self

    @staticmethod
    def init(path=DEFAULT_LOG_PATH):
        """
        static method for initialize the singleton instance
        :param path: path of logs directory
        """
        if AMLogger.shared is not None:
            return

        AMLogger()
        AMLogger.path = path

        # check if log directory exists
        if not os.path.exists(path):
            try:
                os.mkdir(path)
            except OSError:
                print(FontColor.failed(INVALID_LOG_PATH))
                exit(0)

    def get_error_logger(self):
        """
        get error logger instance
        :return: ErrorLogger instance
        """
        ErrorLogger.init(AMLogger.path)
        return ErrorLogger.shared.logger

    def get_access_logger(self):
        """
        get access logger instance
        :return: AccessLogger instance
        """
        AccessLogger.init(AMLogger.path)
        return AccessLogger.shared.logger

    def __error(self, message: str, extra: dict):
        """
        log an error
        :param message: error message
        :param extra: extra params which must be logged
        """
        error_logger = self.get_error_logger()
        if extra is not None and extra.keys():
            error_logger.error(message, extra={'props': extra}, stacklevel=3)
        else:
            error_logger.error(message, stacklevel=3)

    def __info(self, message: str, extra: dict):
        """
        log an info message
        :param message: info message
        :param extra: extra params which must be logged
        """
        access_logger = self.get_access_logger()
        if extra is not None and extra.keys():
            access_logger.info(message, extra={'props' : extra}, stacklevel=3)
        else:
            access_logger.info(message, stacklevel=3)

    @staticmethod
    def error(message: str, extra: dict = {}):
        """
        a static method for calling the AMLogger instance error method
        :param message: error message
        :param extra: extra params which must be logged
        """
        AMLogger.shared.__error(message, extra)

    @staticmethod
    def info(message: str, extra: dict = {}):
        """
        a static method for calling the AMLogger instance info method
        :param message: info message
        :param extra: extra params which must be logged
        """
        AMLogger.shared.__info(message, extra)



from logging import LogRecord, FileHandler

import os


class JSONLogHandler(FileHandler):
    """
    custom json log handler
    """

    def __init__(self, path):
        self.path = path
        self.terminator = ''
        FileHandler.__init__(self, path)

    def emit(self, record: LogRecord):
        """
        the base `emit` method has been overridden for format the `logfile` to JSON file format
        :param record: log record
        """
        if self.is_first_record():
            self.add_opening_bracket()
        else:
            self.remove_closing_bracket()
            self.add_comma()

        super().emit(record)

        self.add_closing_bracket()

    def is_first_record(self):
        """
        check if a log record is first log record or not
        :return: bool result: if a log record is first record return True else return False
        """
        with open(self.path, 'r') as log_handler:
            # get first line of logfile
            content = log_handler.readline()

        size_of_content = len(content)

        return size_of_content == 0

    def add_opening_bracket(self):
        """
        add opening bracket to the log file, formatting json log file
        """
        with open(self.path, 'w') as log_handler:
            log_handler.write('[\n')

    def add_comma(self):
        """
        add comma to (last - 1) log record, formatting json log file
        """
        with open(self.path, 'a') as log_handler:
            log_handler.write(',\n')

    def add_closing_bracket(self):
        """
        add closing bracket to end of logfile, formatting json log file
        """
        with open(self.path, 'a') as log_handler:
            log_handler.write('\n]\n')

    def remove_closing_bracket(self):
        """
        remove closing bracket for add new log record
        """
        with open(self.path, "r+", encoding="utf-8") as file:
            file.seek(0, os.SEEK_END)
            pos = file.tell() - 1

            while pos > 0 and file.read(1) != "\n":
                pos -= 1
                file.seek(pos, os.SEEK_SET)

            if pos > 0:
                file.seek(pos, os.SEEK_SET)
                file.truncate()



import logging
import os
from algomax_common.messages import CLASS_IS_SINGLETON
from .json_log_handler import JSONLogHandler
from .json_log_formatter import JSONLogFormatter
from logquacious.backport_configurable_stacklevel import patch_logger
error_logger = logging.getLogger('error-log')
error_logger.__class__ = patch_logger(error_logger.__class__)


DEFAULT_ERROR_LOG_NAME = 'error.json'


class ErrorLogger:
    """
    custom error logger
    this class is singleton
    """

    shared = None

    def __init__(self):
        if ErrorLogger.shared is not None:
            raise Exception(CLASS_IS_SINGLETON.format('ErrorLogger'))
        else:
            ErrorLogger.shared = self

        self.logger = None

    @staticmethod
    def init(path):
        """
        static method for initialize the singleton instance
        :param path: log directory
        """
        if ErrorLogger.shared is not None:
            return

        ErrorLogger()

        ErrorLogger.shared.logger = error_logger
        ErrorLogger.shared.logger.setLevel(logging.ERROR)

        json_file_handler = JSONLogHandler(os.path.join(path, DEFAULT_ERROR_LOG_NAME))

        file_formatter = JSONLogFormatter()

        json_file_handler.setFormatter(file_formatter)

        ErrorLogger.shared.logger.addHandler(json_file_handler)





class FontColor:
    """
    colors for cli result text
    """

    HEADER = '\033[95m'
    OKBLUE = '\033[94m'
    OKGREEN = '\033[92m'
    WARNING = '\033[93m'
    FAIL = '\033[91m'
    ENDC = '\033[0m'
    BOLD = '\033[1m'
    UNDERLINE = '\033[4m'

    @staticmethod
    def success(message: str):
        """
        get a text and return it with green color
        :param message: an input message
        :return: green color text
        """

        return '{0}{1}{2}'.format(FontColor.OKGREEN, message, FontColor.ENDC)

    @staticmethod
    def failed(message: str):
        """
        get a text and return it with red color
        :param message: an input message
        :return: red color text
        """
        return '{0}{1}{2}'.format(FontColor.FAIL, message, FontColor.ENDC)

    @staticmethod
    def info(message: str):
        """
        get a text and return it with blue color
        :param message: an input message
        :return: blue color text
        """
        return '{0}{1}{2}'.format(FontColor.OKBLUE, message, FontColor.ENDC)

    @staticmethod
    def bold(message: str):
        """
        get a text and return it with bold style
        :param message: an input message
        :return: bold style text
        """
        return '{0}{1}{2}'.format(FontColor.BOLD, message, FontColor.ENDC)



import json
import logging
import traceback
from datetime import datetime


def json_serializer(log):
    """
    serialize log record
    :param log: log record
    :return: serialized log record
    """
    return json.dumps(log, ensure_ascii=False)


def sanitize_log_msg(record):
    """
    sanitize log message
    \n, \r, \t == replace ==> _
    :param record: log record
    :return: sanitized log message
    """
    return record.getMessage().replace('\n', '_').replace('\r', '_').replace('\t', '_')


class JSONLogFormatter(logging.Formatter):
    """
    custom json formatter
    """

    def get_exc_fields(self, record):
        """
        if an exception raised, this method formats the exception message and
        returns { formatted exception message and filename }
        :param record: log record
        :return: a dict => { formatted exception message, filename }
        """
        if record.exc_info:
            exc_info = self.format_exception(record.exc_info)
        else:
            exc_info = record.exc_text
        return {
            'exc_info': exc_info,
            'filename': record.filename,
        }

    @classmethod
    def format_exception(cls, exc_info):
        """
        format exception message
        :param exc_info: an exception that raised
        :return: formatted exception message
        """
        return ''.join(traceback.format_exception(*exc_info)) if exc_info else ''

    def format(self, record):
        """
        format log record to a json object
        :param record: log record
        :return: a json string that contains the log record
        """
        now = datetime.now()
        json_log_object = {
                           "date_time": str(now),
                           "level": record.levelname,
                           "message": sanitize_log_msg(record),
                           "line_number": record.lineno,
                           "module": record.module
                           }
        if hasattr(record, 'props'):
            json_log_object.update({'data': record.props})

        if record.exc_info or record.exc_text:
            json_log_object.update(self.get_exc_fields(record))

        return json_serializer(json_log_object)



from setuptools import setup, find_packages


try:
    from pip._internal.req import parse_requirements
except ImportError:
    from pip.req import parse_requirements


def load_requirements(filename: str):
    requirements = parse_requirements(filename, session='production')
    return [str(requirement.req) for requirement in requirements]


README_PATH = 'README.md'
with open(README_PATH, 'r') as file_reader:
    long_description = file_reader.read()


REQUIREMENT_PATH = 'requirements.txt'

setup(name='algomax-common',
      version='0.1',
      author='mabna DP',
      author_email='mabnadp@gmail.com',
      description='a common package for algomax-cli and algomax-engine',
      long_description=long_description,
      keywords='algomax-common',
      long_description_content_type='text/markdown',
      url='https://mabnadp.com',
      packages=find_packages(),
      classifiers=[
          'Programming Language :: Python :: 3',
          'License :: OSI Approved :: MIT License',
          'Operating System :: OS Independent'
      ],
      install_requires=load_requirements(REQUIREMENT_PATH),
      python_requires='>=3.6'
      )


import logging, os
from .json_log_formatter import JSONLogFormatter
from algomax_common.messages import CLASS_IS_SINGLETON
from .json_log_handler import JSONLogHandler
from logquacious.backport_configurable_stacklevel import patch_logger
access_logger = logging.getLogger('info-log')
access_logger.__class__ = patch_logger(access_logger.__class__)

DEFAULT_ACCESS_LOG_NAME = 'info.json'


class AccessLogger:
    """
    custom access logger
    this class is singleton
    """

    shared = None

    def __init__(self):
        if AccessLogger.shared is not None:
            raise Exception(CLASS_IS_SINGLETON.format('AccessLogger'))
        else:
            AccessLogger.shared = self

        self.logger = None

    @staticmethod
    def init(path):
        """
        static method for initialize the singleton instance
        :param path: log directory
        """
        if AccessLogger.shared is not None:
            return

        AccessLogger()

        AccessLogger.shared.logger = access_logger
        AccessLogger.shared.logger.setLevel(logging.INFO)

        json_file_handler = JSONLogHandler(os.path.join(path, DEFAULT_ACCESS_LOG_NAME))

        file_formatter = JSONLogFormatter()

        json_file_handler.setFormatter(file_formatter)

        AccessLogger.shared.logger.addHandler(json_file_handler)
